#ifndef GLU_INCOMPATIBLE_GL_VERSION
#define GLU_INCOMPATIBLE_GL_VERSION 100903
#endif

#ifdef GLU_VERSION_1_0
    if (!strncmp(name, "GLU_", 4)) {
	i(GLU_SMOOTH)
	i(GLU_FLAT)
	i(GLU_NONE)
	i(GLU_POINT)
	i(GLU_LINE)
	i(GLU_FILL)
	i(GLU_SILHOUETTE)
	i(GLU_OUTSIDE)
	i(GLU_INSIDE)
	i(GLU_BEGIN)
	i(GLU_VERTEX)
	i(GLU_END)
	i(GLU_ERROR)
	i(GLU_EDGE_FLAG)
	i(GLU_CW)
	i(GLU_CCW)
	i(GLU_INTERIOR)
	i(GLU_EXTERIOR)
	i(GLU_UNKNOWN)
	i(GLU_TESS_ERROR1)
	i(GLU_TESS_ERROR2)
	i(GLU_TESS_ERROR3)
	i(GLU_TESS_ERROR4)
	i(GLU_TESS_ERROR5)
	i(GLU_TESS_ERROR6)
	i(GLU_TESS_ERROR7)
	i(GLU_TESS_ERROR8)
	i(GLU_AUTO_LOAD_MATRIX)
	i(GLU_CULLING)
	i(GLU_SAMPLING_TOLERANCE)
	i(GLU_DISPLAY_MODE)
	i(GLU_PATH_LENGTH)
	i(GLU_PARAMETRIC_ERROR)
	i(GLU_DOMAIN_DISTANCE)
	i(GLU_MAP1_TRIM_2)
	i(GLU_MAP1_TRIM_3)
	i(GLU_OUTLINE_POLYGON)
	i(GLU_OUTLINE_PATCH)
	i(GLU_NURBS_ERROR1)
	i(GLU_NURBS_ERROR2)
	i(GLU_NURBS_ERROR3)
	i(GLU_NURBS_ERROR4)
	i(GLU_NURBS_ERROR5)
	i(GLU_NURBS_ERROR6)
	i(GLU_NURBS_ERROR7)
	i(GLU_NURBS_ERROR8)
	i(GLU_NURBS_ERROR9)
	i(GLU_NURBS_ERROR10)
	i(GLU_NURBS_ERROR11)
	i(GLU_NURBS_ERROR12)
	i(GLU_NURBS_ERROR13)
	i(GLU_NURBS_ERROR14)
	i(GLU_NURBS_ERROR15)
	i(GLU_NURBS_ERROR16)
	i(GLU_NURBS_ERROR17)
	i(GLU_NURBS_ERROR18)
	i(GLU_NURBS_ERROR19)
	i(GLU_NURBS_ERROR20)
	i(GLU_NURBS_ERROR21)
	i(GLU_NURBS_ERROR22)
	i(GLU_NURBS_ERROR23)
	i(GLU_NURBS_ERROR24)
	i(GLU_NURBS_ERROR25)
	i(GLU_NURBS_ERROR26)
	i(GLU_NURBS_ERROR27)
	i(GLU_NURBS_ERROR28)
	i(GLU_NURBS_ERROR29)
	i(GLU_NURBS_ERROR30)
	i(GLU_NURBS_ERROR31)
	i(GLU_NURBS_ERROR32)
	i(GLU_NURBS_ERROR33)
	i(GLU_NURBS_ERROR34)
	i(GLU_NURBS_ERROR35)
	i(GLU_NURBS_ERROR36)
	i(GLU_NURBS_ERROR37)
	i(GLU_INVALID_ENUM)
	i(GLU_INVALID_VALUE)
	i(GLU_OUT_OF_MEMORY)
 	i(GLU_INCOMPATIBLE_GL_VERSION)

#ifdef GLU_VESION_1_1
	i(GLU_VERSION)
	i(GLU_EXTENSIONS)
#endif

#ifdef GLU_VERSION_1_1
	i(GLU_SAMPLING_METHOD)
	i(GLU_PARAMETRIC_TOLERANCE)
	i(GLU_U_STEP)
	i(GLU_V_STEP)
#endif

#ifdef GLU_VERSION_1_2
	i(GLU_TESS_ERROR)
	i(GLU_TESS_BEGIN)
	i(GLU_TESS_EDGE_FLAG)
	i(GLU_TESS_VERTEX)
	i(GLU_TESS_COMBINE)
	i(GLU_TESS_END)
	i(GLU_TESS_ERROR_DATA)
	i(GLU_TESS_BEGIN_DATA)
	i(GLU_TESS_EDGE_FLAG_DATA)
	i(GLU_TESS_VERTEX_DATA)
	i(GLU_TESS_COMBINE_DATA)
	i(GLU_TESS_END_DATA)
	i(GLU_TESS_WINDING_RULE)
	i(GLU_TESS_WINDING_ODD)
	i(GLU_TESS_WINDING_NONZERO)
	i(GLU_TESS_WINDING_POSITIVE)
	i(GLU_TESS_WINDING_NEGATIVE)
	i(GLU_TESS_WINDING_ABS_GEQ_TWO)
# endif

/* 75 */
#ifdef GLU_EXT_object_space_tess
	i(GLU_OBJECT_PARAMETRIC_ERROR_EXT)
	i(GLU_OBJECT_PATH_LENGTH_EXT)
#endif

	}
	else
#endif
